
# Maybe for backwards compatibility?
try:
    set
except NameError:
    from sets import Set as set


class Event(object):
    """
    Event object generated by mediator.
    """
    def __init__(self, name, **kwArgs):
        self.name = name
        for key,value in kwArgs.items():
            setattr(self, key, value)
        self.mediator = None
    
    def __repr__(self):
        return "<Event '" + self.name + "'>"
        
class Mediator(object):
    def __init__(self):
        """
        Create a mediator. (You probably want to use the Singleton mediator
        instead.)
        """
        self.listeners = {}

    def addListener(self, evtName, listener):
        """
        Add a listener to this mediator

        `evtName`  the name of the event to listen for
        `listener` a callable object to call when an event happens. The event
                   object will be passed to this callable
        """
        self.listeners[evtName] = (self.listeners.get(evtName, set())
            .union(set((listener,))))
        
    def removeListener(self, evtName, listener):
        """
        Remove a listener from this mediator
        
        `evtName`  the name of the event to no longer listen for
        `listener` a callable object to call when an event happens
        """
        self.listeners[evtName] = (self.listeners.get(evtName, set())
            .difference(set((listener,))))

    def publishEvent(self, event, **kwArgs):
        """
        Publish an event and notify any listeners who want to be told
        about it.

        `event` Event object to publish
        """
        if (type(event) is str):
            event = Event(event, **kwArgs)
        event.mediator = self
        for listener in self.listeners.get(event.name, []):
            try:
                #print "dispatch '" + str(event) + "' to " + listener.__name__
                pass
            except:
                pass
            listener(event)

# Singleton mediator
mediator = Mediator()

if __name__ == '__main__':
    l = []
    
    def c1(evt):
        l.append('c1')
    def c2(evt):
        l.append('c2')
        
    m = Mediator()
    m.addListener('e1', c1)
    m.addListener('e1', c2)
    m.publishEvent('e1', foo='bar')
    assert ('c1' in l)
    assert ('c2' in l)
    
    l = []
    m.removeListener('e1', c1)
    m.publishEvent(Event('e1', foo='bar'))
    assert ('c2' in l)
    assert not ('c1' in l)
