#!/usr/bin/env python

# Village
DEFAULT_NUM_HUTS=30

import os
import pygame
import math
import random
import events
import hut
from pygame.locals import *
from mediator import mediator

class Village(object):

    def __init__(self, position, radius, numHuts=DEFAULT_NUM_HUTS):
        """
        Create a new Village

        `position` 2-tuple (X, Y) of the village center
        `radius` radius of the village
        `numHuts` number of huts in the village
        """
        
        super(Village, self).__init__()
        self._hut_array = []
        self._center = position
        self._radius = radius
        self._num_huts = numHuts
        
        # Create the Huts
        length_hut_diag = int(math.sqrt(hut.SIZE/2*hut.SIZE/2 + hut.SIZE/2*hut.SIZE/2))
        hut_angle = 0
        hut_radius = random.randint(0, self._radius) 
       
        for i in range(0, self._num_huts):

            # Calculate Hut X
            #unit_point_x = hut_radius * math.cos(math.radians(hut_angle))
            unit_point_x = hut_radius * math.cos(math.radians(hut_angle))
            hut_x = self._center[0] + unit_point_x

            # Calculate Hut Y
            hut_y = self._center[1] + (unit_point_x * math.tan(math.radians(hut_angle)))

            self._hut_array.append(hut.Hut((hut_x, hut_y))) 

            # Change angle
            hut_angle = 360*(i+1)/self._num_huts

            # Change radius
            max_radius_of_last_hut = hut_radius + length_hut_diag
            min_radius_of_last_hut = hut_radius - length_hut_diag
            if (self._radius - max_radius_of_last_hut < min_radius_of_last_hut - length_hut_diag):
                hut_radius = random.randint(length_hut_diag, min_radius_of_last_hut + 1)
            else:
                hut_radius = random.randint(max_radius_of_last_hut, self._radius + 1)
        
    def huts():
        doc = "Hut Array"
        def fget(self):
            return self._hut_array
        return locals()
    huts = property(**huts())

    def health():
        doc = "Village Health"
        def fget(self):
            health = 0
            for hut in self._hut_array:
                health += hut.health 
            return int(health/10)
        return locals()
    health = property(**health())
